#
# Licence GPL v3.
#
# Copyright Chris Johns (chrisj@rtems.org)
#
import pq
import os
import shutil
import time
import urlparse

cpukit_dirs = [ 'rtems', 'posix', 'score', 'sapi', 'itron', 'include',
                'libcsupport', 'libnetworking', 'libmisc', 'libblock', 'libfs',
                'libi2c', 'shttpd', 'httpd', 'librpc', 'ftpd', 'pppd', 'shttpd',
                'httpd', 'libgnat', 'libmd', 'telnetd', 'zlib' ]

def build_output_basename(bsp):
    return 'stdout-' + bsp + '.log'

def build_output_suffix():
    return '.xz'

def build_output_archive(bsp):
    return build_output_basename(bsp) + build_output_suffix()

def build_output_config_log():
    return 'config.log'

def build_output(workspace, target, bsp):
    return os.path.join(workspace, target, build_output_basename(bsp))

def build_output_tail():
    return 50

def source_find(file, target, bsp, source):
    sp = file.split(os.sep)
    if len(sp) > 3:
        skip = len(sp)
        for p in range(len(sp) - 1, 0, -1):
            skip -= 1
            if sp[p] == 'include':
                if os.path.splitext(file)[1] == '.inl':
                    sp[p] = 'inline'
                break
        t = os.path.join(source, 'cpukit/score/cpu', target, 
                         apply(os.path.join, sp[skip + 1:]))
        if os.path.isfile(t):
            file = t
        else:
            f = apply(os.path.join, sp[skip:])
            for d in cpukit_dirs:
                t = os.path.join(source, 'cpukit', d, f)
                if os.path.isfile(t):
                    file = t
                    break
    return file

def title():
    return 'RTEMS Build Results'

def target_title_page(target, bsp, page):
    return 'Target: %s  Page:%d ' % (pq.target_bsp_name(target, bsp, 3), page)

def target_title(target, bsp):
    return 'Target: %s ' % (pq.target_bsp_name(target, bsp, 3))

def help_url():
    return '<small><br><a href="http://www.rtems.org/wiki/index.php/Build_Results">Help and details</a></small>'

def errors_per_page():
    return 30

def history_records():
    return 14

def stylesheet():
    return 'rtems/style.css'

def src_tarball(date):
    return src_prefix() + date + src_suffix()

def src_prefix():
    return 'rtems-cvs-'

def src_suffix():
    return '.tar.bz2'

def publish_config_setup(base, output):
    dst = os.path.join(output, 'rtems')
    if not os.path.exists(dst) or not os.path.isdir(dst):
        os.makedirs(dst)
    try:
        shutil.copy(os.path.join(base, 'pq', 'rtems', 'style.css'), dst)
    except IOError, e:
        pass

def publish_source_file(source, line):
    if os.path.isabs(source):
        return source
    return '<a href="' + \
        urlparse.urljoin('http://www.rtems.org/cgi-bin/cvsweb.cgi/rtems/', source) + \
        '?rev=.;ln=1#l' + str(line) + '" target="_blank">' + '%s:%d' % (source, line) + '</a>'

def publish_filename(target, bsp, t):
    if type(t) == time.struct_time:
        t = time.strftime('%Y%m%d', t)
    elif type(t) == float:
        t = time.strftime('%Y%m%d', time.localtime(t))
    return '%s-%s-%s' % (target, bsp, t)
