#
# Licence GPL v3.
#
# Copyright Chris Johns (chrisj@rtems.org)
#
import optparse
import os
import pq
import pqconfig
import re
import shutil
import sys
import tailer
import time

class publish:
    def __init__(self, name, outpath, workspace, date):
        self.output = open(os.path.join(outpath, name), 'w') 
        self.name = name
        self.date = date
        self.time = time.mktime(time.strptime(date, '%Y%m%d'))
        self.outpath = outpath
        self.workspace = workspace

    def __del__(self):
        if self.output:
            self.output.close()

    def write(self, s):
        print >> self.output, s,

    def writeln(self, s):
        print >> self.output, s

    def file_exists(self, name):
        return os.path.isfile(os.path.join(self.outpath, name))

    def html_header(self, title):
        self.writeln('<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 3.2 Final//EN">')
        self.writeln('<html>')
        self.writeln(' <head>')
        self.writeln('  <title>' + title + '</title>')
        self.writeln('  <meta http-equiv="Content-Language" content="English" />')
        self.writeln('  <meta http-equiv="Content-Type" content="text/html; charset=us-ascii" />')
        ss = pqconfig.stylesheet()
        if ss:
            self.writeln('  <link rel="stylesheet" type="text/css" href="' + ss + \
                         '" media="screen" />')
        self.writeln(' </head>')
        self.writeln(' <body>')

    def html_footer(self):
        self.writeln(' </body>')
        self.writeln('</html>')

    def heading(self, heading):
        self.writeln('  <div class="heading">')
        self.writeln('   ' + heading)
        self.writeln('   <div class="datetime">' + time.ctime() + '</div>')
        self.writeln('   ' + pqconfig.help_url())
        self.writeln('  </div>')
        
    def footer(self):
        self.writeln('  <div class="footer">')
        self.writeln('   <br><br>&copy Chris Johns 2010')
        self.writeln('  </div>')

    def table_open(self, _class, cellspacing = 0):
        self.writeln('  <table class="' + _class + '" cellspacing="' + \
                     str(cellspacing) + '" cellpadding="2">')

    def table_close(self):
        self.writeln('  </table>')

class publish_errors(publish):
    def __init__(self, title, name, parent, outpath, workspace, date):
        publish.__init__(self, name, outpath, workspace, date)
        self.html_header(title)
        self.heading(title)
        self.parent = parent

    def __del__(self):
        self.footer()
        publish.__del__(self)

    def navbar(self, first, prev, next, last):
        self.writeln('  <div class="navbar">')
        self.writeln('   <a href="' + self.parent + '">Summary</a>' + \
                         '&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;')
        if first:
            self.writeln('   <a href="./' + first + '">First</a>')
        else:
            self.writeln('   First')
        self.writeln('   &nbsp;&nbsp;&nbsp;&nbsp;')
        if prev:
            self.writeln('   <a href="./' + prev + '">Prev</a>')
        else:
            self.writeln('   Prev')
        self.writeln('   &nbsp;&nbsp;&nbsp;&nbsp;')
        if next:
            self.writeln('   <a href="./' + next + '">Next</a>')
        else:
            self.writeln('   Next')
        self.writeln('   &nbsp;&nbsp;&nbsp;&nbsp;')
        if last:
            self.writeln('   <a href="./' + last + '">Last</a>')
        else:
            self.writeln('   Last')
        self.writeln('  </div>')

    def error_header(self):
        self.write('   <tr class="error-table-top">')
        self.writeln('<th>Dups</th><th>E</th><th>File</th><th>Error/Warning</th></tr>')
        
    def error(self, error, on):
        if on:
            s = '    <tr class="error-table-on">'
        else:
            s = '    <tr class="error-table-off">'
        s += '<td class="error-table-dups">%d</td>' % (error['dup'])
        if error['error'].type == 'error':
            s += '<td class="error-table-error">&nbsp;</td>'
        else:
            s += '<td class="error-table-warning">&nbsp;</td>'
        source_file = pqconfig.publish_source_file(error['error'].file, error['error'].line)
        if source_file:
            s += '<td>' + source_file + '</td>'
        else:
            s += '<td>%s:%d</td>' % (error['error'].file, error['error'].line)
        s += '<td>' + error['error'].info + '</td>'
        self.writeln(s + '</tr>')
        
class publish_tail(publish):
    def capture(self, log, lines):
        self.tail = tailer.tail(log, lines)

    def write_page(self, title):
        self.html_header(title)
        self.heading(title)
        self.writeln('<pre class="build-output">')
        for l in self.tail:
            self.writeln(l)
        self.writeln('</pre>')
        self.footer()

class publish_bsp:
    def __init__(self, bsp, parent, base, outpath, workspace, target, date):
        self.bsp = bsp
        self.parent = parent
        self.base = base
        self.errors = None
        self.tail = None
        self.outpath = outpath
        self.workspace = workspace
        self.target = target
        self.date = date
        self.title = pqconfig.target_title(self.target, self.bsp)
        self.filename_base = pqconfig.publish_filename(self.target,
                                                       self.bsp,
                                                       self.date)
        self.tail = publish_tail(self.filename_base + '-tail.html',
                                 self.outpath,
                                 self.workspace,
                                 self.date)

    def copy_config_log(self):
        cfg = os.path.join(self.workspace,
                           self.target, 
                           pqconfig.build_output_config_log())
        if not os.path.isfile(cfg):
            print >> sys.stderr, "cannot find the config log:", cfg
        else:
            try:
                dst = os.path.join(self.outpath, 
                                   self.filename_base + '-config.log')
                shutil.copy(cfg, dst)
            except IOError, e:
                print >> sys.stderr, "Cannot copy config log:", e
                pass

    def copy_archive(self):
        out = os.path.join(self.workspace,
                           self.target,
                           pqconfig.build_output_archive(self.bsp))
        if not os.path.isfile(out):
            print >> sys.stderr, "cannot find the output archive:", out
        else:
            try:
                dst = os.path.join(self.outpath, 
                                   self.filename_base + '-output' + \
                                       pqconfig.build_output_suffix())
                shutil.copy(out, dst)
            except IOError, e:
                print >> sys.stderr, "Cannot copy output archive:", e
                pass

    def load_errors(self, data, source):
        self.errors = None
        try:
            self.errors = pq.errors(self.base, data, source, pqconfig.source_find)
            log = open(os.path.join(self.workspace, self.target,
                                    'stdout-'  + self.bsp + '.log'))
            self.errors.load_errs(self.target, self.bsp, log)
            self.tail.capture(log, pqconfig.build_output_tail())
            log.close()
        except IOError, e:
            print >> sys.stderr, "cannot open the BSP log:", self.target + ':' + self.bsp
            pass

    def write_pages(self, history, data, source):
        page = None
        count = 0
        page_num = 0
        pages = (self.errors.count() / pqconfig.errors_per_page())
        first_filename = self.filename_base + '-1.html'
        last_filename = self.filename_base + '-%d.html' % (pages)
        prev_filename = None
        next_filename = None
        filename = None
        for error in self.errors.walk_errs():
            if self.errors.check_all(self.target, error['error'].error):
                continue
            if not page:
                count = 0
                page_num += 1
                filename = self.filename_base + '-%d.html' % (page_num)
                page = publish_errors(pqconfig.target_title_page(self.target, 
                                                                 self.bsp,
                                                                 page_num),
                                      filename,
                                      self.parent,
                                      self.outpath,
                                      self.workspace,
                                      self.date)
                page.table_open('error-table')
                page.error_header()
                table_on = False
            page.error(error, table_on)
            table_on = not table_on
            count += 1
            if count >= pqconfig.errors_per_page():
                page.table_close()
                page.navbar(first_filename, prev_filename,
                            next_filename, last_filename)
                page = None
                prev_filename = filename
                if page < pages:
                    next_filename = \
                        self.filename_base + '-%d.html' % (page_num + 1)
                else:
                    next_filename = None
        page = None
        self.tail.write_page(self.title)
        self.copy_config_log()
        self.copy_archive()

class publish_target(publish):
    def __init__(self, base, outpath, workspace, target, date):
        publish.__init__(self, target + '.html',
                         outpath, workspace, date)
        self.base = base
        self.bsp_list = []
        self.bsps = {}
        self.title = 'Target: ' + pq.target_name(target, 2)
        self.target = target
        self.html_header(self.title)
        self.heading(self.title)
        self.writeln( '<a href=".">Targets</a>')

    def __del__(self):
        self.writeln( '<a href=".">Targets</a>')
        self.footer()
        publish.__del__(self)

    def get_bsp_list(self):
        ls = os.listdir(os.path.join(self.workspace, self.target))
        of = re.compile('stdout-.+\.log\.xz')
        for f in ls:
            if of.match(f):
                self.bsp_list.append(f[7:-7])
                pass

    def write_pages(self, history, data, source):
        self.bsp_list = pq.bsp_list(self.workspace, self.target)
        if not self.bsp_list:
            return
        self.bsps = {}
        for bsp in self.bsp_list:
            self.bsps[bsp] = publish_bsp(bsp,
                                         self.name,
                                         self.base,
                                         self.outpath,
                                         self.workspace,
                                         self.target,
                                         self.date)
            self.bsps[bsp].load_errors(data, source)
            self.bsps[bsp].write_pages(history, data, source)
            history.add(self.workspace,
                        self.target,
                        bsp,
                        self.bsps[bsp].errors,
                        self.date)
            history.remove_oldest(self.target,
                                  bsp)
            self.bsp_stats(history, bsp)

    def bsp_stats(self, history, bsp):
        self.table_open('stats-table', 1)
        self.writeln('   <tr class="stats-table-target"><td colspan="8">' + \
                         pq.target_bsp_name(self.target, bsp, 2) + \
                         '</td></tr>')
        self.writeln('   <tr class="stats-table-top"><th>Date</th>' + \
                         '<th>Errors</th><th>Warnings</th><th>Notes</th>' + \
                         '<th>Duplicates</th><th>Status</th><th>Results</th>' + \
                         '<th>Output</th></tr>')
        for h in history.history:
            if h[0] == self.target and h[1] == bsp:
                t = history.date2time(h[2])
                filename_base = pqconfig.publish_filename(h[0], h[1], t)
                src_tarball = pqconfig.src_tarball(h[2])
                self.write('   <tr class="stats-table-row">' + \
                               '<td align="center">')
                if self.file_exists(src_tarball):
                    d = time.strftime('%d %b %Y',
                                      time.localtime(history.date2time(h[2])))
                    self.write('<a href="' + src_tarball + '">' + d + '</a>')
                else:
                    self.write(h[2])
                self.write('</td>')
                self.write('<td>' + h[3] + '</td>')
                self.write('<td>' + h[4] + '</td>')
                self.write('<td>' + h[5] + '</td>')
                self.write('<td>' + h[6] + '</td>')
                if h[7] == 'PASSED':
                    _class = 'stats-target-good'
                else:
                    _class = 'stats-target-bad'
                self.write('<td class="' + _class + '">' + h[7] + '</td>')
                self.write('<td class="stats-target-results">')
                name = filename_base + '-1.html'
                if self.file_exists(name):
                    self.write('<a href="' + name + '">Results</a>')
                else:
                    self.write(' ')
                self.write('</td><td align="center">')
                name = filename_base + '-tail.html'
                if self.file_exists(name):
                    self.write('<a href="' + name + '">Output</a>')
                else:
                    self.write(' ')
                config = filename_base + '-config.log'
                archive = filename_base + '-output' + pqconfig.build_output_suffix()
                if self.file_exists(config) or self.file_exists(archive):
                    self.write('&nbsp;<small><i>(')
                    if self.file_exists(config):
                        self.write('<a href="' + config + '">config</a>')
                        if self.file_exists(archive):
                            self.write(',')
                    if self.file_exists(archive):
                        self.write('<a href="' + archive + '">full</a>')
                    self.write(')</i></small>')
                self.writeln('</td></tr>')
        self.table_close()

    def numof_bsps(self):
        return str(len(self.bsps))

    def bsp_count(self, which):
        count = 0
        for bsp in self.bsps:
            if which == 'dups':
                count += self.bsps[bsp].errors.dup_count
            else:
                c = self.bsps[bsp].errors.counts[which]
                if which == 'error' and c == 0:
                    if pq.status(self.workspace, self.target, bsp) != 'PASSED':
                        c = 1
                count += c
        return str(count)

    def bsp_errors(self):
        return self.bsp_count('error')

    def bsp_warnings(self):
        return self.bsp_count('warning')

    def bsp_notes(self):
        return self.bsp_count('note')

    def bsp_dups(self):
        return self.bsp_count('dups')

    def status(self, history):
        state = 'PASSED'
        for h in history.history:
            if h[0] == self.target:
                if h[7] != 'PASSED':
                    if h[2] == self.date:
                        return 'FAILED'
                    state = 'HAS FAILED'
        return state

def make_outpath(outpath):
    if not os.path.exists(outpath) or \
            not os.path.isdir(outpath):
        os.makedirs(outpath)

def copy_source_tarball(outpath, workspace, date):
    src = os.path.join(workspace, pqconfig.src_tarball(date))
    if not os.path.isfile(src):
        print >> sys.stderr, "cannot find the source tarball:", src
    else:
        try:
            shutil.copy(src, outpath)
        except IOError, e:
            print >> sys.stderr, "Cannot copy source tarball:", e
            pass

class publish_results(publish):
    def __init__(self, label, base, data, workspace, source, targets, outpath, date):
        publish.__init__(self, 'index.html', outpath, workspace, date)
        self.history = pq.history(data, outpath)
        self.base = base
        self.data = data
        self.source = source
        self.targets = targets
        self.target = {}
        self.title = 'RTEMS Builder Results: ' + label
        self.html_header(self.title)

    def __del__(self):
        self.footer()
        self.history.unload(self.data)
        publish.__del__(self)

    def write_pages(self):
        target = {}
        self.heading(self.title)
        for target in self.targets:
            self.target[target] = publish_target(self.base, self.outpath, 
                                                 self.workspace, target, self.date)
            self.target[target].write_pages(self.history, self.data, self.source)
        self.table_open('stats-table', 1)
        self.writeln('   <tr class="stats-table-target"><td colspan="7">Targets</td></tr>')
        self.writeln('   <tr class="stats-table-top"><th>Target</th>' + \
                         '<th>BSP Count</th><th>Errors</th><th>Warnings</th>' + \
                         '<th>Notes</th><th>Duplicates</th><th>Status</th></tr>')
        for target in self.targets:
            self.writeln('   <tr class="stats-table-row">' + \
                             '<td align="center">' + \
                             '<a href="' + target +'.html">' + \
                             pq.target_name(target, 2) + \
                             '</a></td>')
            self.writeln('   <td>' + self.target[target].numof_bsps() + '</td>')
            self.writeln('   <td>' + self.target[target].bsp_errors() + '</td>')
            self.writeln('   <td>' + self.target[target].bsp_warnings() + '</td>')
            self.writeln('   <td>' + self.target[target].bsp_notes() + '</td>')
            self.writeln('   <td>' + self.target[target].bsp_dups() + '</td>')
            state = self.target[target].status(self.history)
            if state == 'PASSED':
                _class = 'stats-target-good'
            elif state == 'FAILED':
                _class = 'stats-target-bad'
            else:
                _class = 'stats-target-was-bad'
            self.writeln('   <td class="' + _class + '">' + state + '</td>')
            self.writeln('   </tr>')
        self.table_close()

#
# Command line options.
#
cl_options = [
    optparse.make_option('-o', action = 'store', type = 'string', 
                         dest = 'outpath', default = None,
                         help = 'Directory to write the HTML output to'),
    optparse.make_option('-D', action = 'store', type = 'string', 
                         dest = 'date', default = None,
                         help = 'Date of the source tarball'),
    optparse.make_option('-L', action = 'store', type = 'string', 
                         dest = 'label', default = 'User',
                         help = 'The build label'),
    # Standard options
    optparse.make_option('-d', action = 'store', type = 'string', 
                         dest = 'data', default = os.environ['HOME'],
                         help = 'Specify the data directory'),
    optparse.make_option('-w', action = 'store', type = 'string', 
                         dest = 'workspace', default = None,
                         help = 'Specify the workspace directory'),
    optparse.make_option('-s', action = 'store', type = 'string', 
                         dest = 'source', default = None,
                         help = 'Specify the source directory'),
    optparse.make_option('-t', action = 'store', type = 'string', 
                         dest = 'targets', default = None,
                         help = 'List of comma seperated targets'),
    optparse.make_option('-c', action = 'store', type = 'string',
                         dest = 'config',
                         default = os.environ['HOME'] + '/.pq-config',
                         help = 'Configration file name to read')
]

def user_main(base, argv):
    (options, config, data, workspace, source, targets) = \
        pq.parse_std_args(argv, cl_options)
    if not options.outpath:
        print >> sys.stderr, 'no output directory specified'
        sys.exit(1)
    if not options.date:
        print >> sys.stderr, 'no source tarball date specified'
        sys.exit(1)

    pqconfig.publish_config_setup(base, options.outpath)
    make_outpath(options.outpath)
    copy_source_tarball(options.outpath, workspace, options.date)

    results = publish_results(options.label, 
                              base, data, workspace, source, targets,
                              options.outpath, options.date)
    results.write_pages()
    del results
